%% McBASC score
%input: alignment file: aln
%       substitution matrix: matrix - - 'McLachlan' or 'BLOSUM62' or
%       Dayhoff or Gonnet or Pam50 
%       output filename: out
%output: score for each pair of columns
%        i j score
function score = BASCscore(alignment, matrix, out, proteinname, temppath, seprate_site)
clear substitution_matrix;
substitution_matrix = getSubMatrix(matrix);
num_seq = size(alignment,1);
num_column = size(alignment(1,1).Sequence, 2);
mapped_index = -1 * ones(num_seq, num_column);
aminonames = {'R', 'N', 'D', 'C', 'Q', 'E', 'G', 'H', 'I', 'L', 'K', 'M', 'F', 'P', 'S', 'T', 'W', 'Y', 'V', 'A'};
fiddata = fopen([temppath, 'IndexMSA20AA', proteinname,'.mat']);
if fiddata == -1    
    for i = 1 : num_seq
        for j = 1 : num_column
            mapped_index20(i,j) = MapResiduesToIndex20(alignment(i,1).Sequence(j), aminonames);
        end
    end
    save([temppath, 'IndexMSA20AA', proteinname,'.mat'], 'mapped_index20');
    mapped_index = mapped_index20;
else
    fclose(fiddata);
    clear tempdata;
    tempdata = load([temppath, 'IndexMSA20AA', proteinname,'.mat']);
    mapped_index = tempdata.mapped_index20;
    clear tempdata;    
end    

scoreMatrix = substitution_matrix.score;
% compute the matrix for each column
fiddata = fopen([temppath, 'SubstitutionScore', proteinname,'.mat']);
if fiddata == -1    
    temp = zeros(1,num_seq*(num_seq-1)/2);
    for i = 1 : num_column
        % comparison between each pasir of sequence
        t = 1;
        for j = 1 :num_seq-1
            for k = j+1 : num_seq
                % if there is a gap in the position, then the score is -100
                % and it will not be considered when the correlated
                % coefficients are being calculated
                if mapped_index(j,i) > 0 && mapped_index(k,i) > 0
                    temp(t) = scoreMatrix(mapped_index(j,i),mapped_index(k,i));
                else
                    temp(t) = -100;
                end
                t = t + 1;
            end
        end
        t = t - 1;
        distances{i} = temp(1,1:t);
    end
    clear temp;
    save([temppath, 'SubstitutionScore', proteinname,'.mat'], 'distances');
else
    fclose(fiddata);
    clear tempdata;
    tempdata = load([temppath, 'SubstitutionScore', proteinname,'.mat']);
    distances = tempdata.distances;
    clear tempdata;
end
    
score = zeros(num_column, num_column);
% calculate the correlated coefficients
index = zeros(seprate_site*(num_column-seprate_site),2);
t = 0;
for i = 1 : seprate_site
    for j = seprate_site+1 : num_column   
        t = t + 1;
        index(t,1) = i;
        index(t,2) = j;                      
    end
end
temp = zeros(1,seprate_site*(num_column-seprate_site));
parfor i = 1 : size(index,1)
    temp(i) = getScore(distances{index(i,1)}, distances{index(i,2)});
end

t = 0;
% fid = fopen(out,'w');
% fprintf(fid, 'i  j   score\n');
for i = 1 : seprate_site
    for j = seprate_site+1 : num_column
        t = t + 1;
        score(i,j) = temp(t);
        score(j,i) = score(i,j);
%         fprintf(fid, '%d\t%d\t%18.12f\n', i-1,j-1,score(i,j));
    end
end
% fclose(fid);







function score = getScore(distancesI, distancesJ)
if size(distancesI,2) ~= size(distancesJ,2)
    'error in getScore'
    pause();
end
sum_x = 0; 
sum_y = 0;
sum_x_squ = 0;
sum_y_squ = 0;
sum_xy = 0;
t = 0;
for i = 1 : size(distancesI,2)
    if distancesI(i) ~= -100 && distancesJ(i) ~= -100
        sum_xy = sum_xy + distancesI(i) * distancesJ(i);
        sum_x = sum_x + distancesI(i);
        sum_y = sum_y + distancesJ(i);
        sum_x_squ = sum_x_squ + distancesI(i) * distancesI(i);
        sum_y_squ = sum_y_squ + distancesJ(i) * distancesJ(i); 
        t = t + 1; % count
    end
end
temp = sum_xy - sum_x * sum_y / t;
score = temp / (sum_x_squ - sum_x * sum_x / t);
score = score * temp / (sum_y_squ - sum_y * sum_y / t);
if isnan(score) || isinf(score)
    score = 0;
end


