% demo for using the iPatch program
% Written by Qiang LUO, 2009-3-2
% require the program of PSA running in Unix
% contact: mrqiangluo@gmail.com

%% quick information of this demo
% This demo can be run by typing 'Demo' and enter at the command window.
% The results will be generated in the result folder for example 1.  By
% commenting and uncommenting the corresponding codes in the input and output
% sections for example 2 and example 3,  this demo can run on those examples.
% More details about this package please refer to the readme.pdf included
% in this package.

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Paramters Setting Section  
%
% Please set the parameters for your proteins
%
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%
%   For INPUT    %
%%%%%%%%%%%%%%%%%%

%% example 1
MSAfile = 'CheAP1CheY6.aln';    % the name of the multiple sequence alignment file for the paired protein sequences.
                                                % Note that there must be a sequence with the header specified as '>referece' in this MSA
pdbfile1 = '1TQG.pdb';                % the PDB file for one protein
pdbfile2 = '1TMY.pdb';                % the PDB file for the other protein
jobname = 'CheACheY';              % whatever jobname you like. this name will be used as a sub-string in the names of the 
                                                % result files as well as some processing files
segments.number{1} = [1,104];   % predictions are made on the domains specified here by residue sequence
                                                % numbers. The residue sequence numbers are defined as those in the PDB file.                                                
segments.number{2} = [2,119];
segments.chain = {'A','A' };          % the chain ID's of the proteins interested
location{1} = [1,105];                   % The first domain defined by the residue sequence numbers and chain ID's above 
                                                 % is located in the regions specified by location{1}, and the second domain is in
                                                 % the region specified by location{2}.                                                 
location{2} = [106,225];


%% example 2
% % get the pairing alignment ready with the reference sequence in it
% MSAfile = '2CH4_seqs_nr90.aln';
% pdbfile1 = '2CH4.pdb';
% pdbfile2 = '2CH4.pdb';
% jobname = '2ch4';
% segments.number{1} = [541,670]; 
% segments.number{2} = [1,151];
% segments.chain = {'B','Y' };
% location{1} = [1,130];  
% location{2} = [131, 281];

%% example 3 
% MSAfile = '1F51_seqs_nr90.aln';   
% pdbfile1 = '1F51.pdb';                   
% pdbfile2 = '1F51.pdb';                   
% jobname = '1f51';
% segments.number{1} = [11,192]; 
% segments.number{2} = [1203,1321];
% segments.chain = {'A','E' };
% location{1} = [1,182]; % the number of residues of the first protein on the  reference sequence in the MSA
% location{2} = [183, 301];                        


%%%%%%%%%%%%%%%%%
%  For OUTPUT   %
%%%%%%%%%%%%%%%%%
%% for top N predictions
% you can choose the top N across two proteins or the top N on each protein
scoreID = 5;   % use the iPatch to make predictions, or you can change from 1 to 5 to use different scores, 
                     % including APro, PPro, TPro, EBMcBASC, and iPatch
topmode = 0;  % top scores in two proteins; 
                     % if topmode = 1, then the predictions will be made as the top N predictions on each protein
topN = 40;      % top N predictions
%% for score cutoff
% you can either choose the score cutoff as percentil  of the score by setting cutoff to -1
% or by set some actual numbers like 0.5 as the cutoff.
cutoff = -1; percentil = 90;

%% choose which sequences you need to see in Jalview 
predictions_selection_criterion = 1; % the topN prediction results will be marked on the sequence; if set it to 0,
                                                    % the predictions above the score cutoff will be marked
% specify the sequences that will be shown in Jalview by giving the headers
seq_header{1} = 'reference';    % the contact sites must be mapped on the reference sequence
seq_header{2} = 'gi|12646244521436676|gb|none|ref';  % some other  sequences in the MSA file can also be displayed in Jalview
% seq_header{1} = 'gi|1588960915889609|gb|none|ref';
% seq_header{2} = 'gi|1588739115887392|gb|none|ref';




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Detailed parameters for iPatch algorithm %
% these are the default settings below           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear parameters;
parameters.surfaceoption = 0; % if set to 0,the surface information for the domain interested is
                              % given by running the psa on the domain; if set it to
                              % 1, the surface information for the domain interested is
                              % given by running the psa on the whole protein on
                              % which this domain is located;
parameters.gapcutoff = 0.5;   % columns with gap percentage greater than this cutoff wont be used in the score calculation
parameters.discutoff = 4.5;   % the intra domain space neighbors are defined by this cutoff, i.e., if the distances 
                              % between residues that are measured as the
                              % shortest distance between any nonhydrogen atoms
                              % are less than this cutoff, these two
                              % residues are intra domain neighbors
parameters.parallelized = 1;  % 1, if you can run the matlabpool, most of the algorithms can be
                              % parallelized; 0, otherwise.
parameters.randomized = 0;    % 1, if you want to get the scores for the random matching pairs as backgroud score; 0, otherwise.
parameters.reference = 0;     % 1, if you want to include your reference sequence in the score calculation; 0, otherwise.  
parameters.cleancache = 1;    % clean all the cache files; if set to be 0, the cache files will be used, if any
parameters.modifiedres = 1;   % if 1, change MSE to MET, CSE to CSC; if 0, those modified residues will be ignored.
parameters.manualalignment = 0; % if 0, the alignment between the pdb sequence and the reference sequence 
                                % will be generated by using the local alignment algorithm swalign(); 
                                % if 1, the user specified alignment will be used for the residue number 
                                % assignment 
                                                             
                                
                                
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Calculating scores  
%
%
%
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% calling the iPatch
result = iPatch(pdbfile1, pdbfile2, MSAfile, location, segments, jobname, parameters);
% output data structure
% result.pairwisescores{1} pairPPro  
% result.pairwisescores{2} pairTPro  
% result.pairwisescores{3} pairMcBASC  
% result.sitescores{1} APro 
% result.sitescores{2} PPro
% result.sitescores{3} TPro
% result.sitescores{4} EBMcBASC
% result.sitescores{5} Comb 
% result.protein  a structure data format with many fields

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Display the results 
%
%
%
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% find topN predictions for a given score and generat the output file
predictions_top = toppredictions(topN, topmode, result, scoreID);
% the results are printed in ./result_topN.txt
outputresultintofile(['./result/result_',result.scorenames{scoreID},num2str(topN), jobname, '.txt'], result, predictions_top);

%% display the score distribution 
displayscoredistribution(result, scoreID);

%% find topN predictions for a given score by score cutoff choosen by users
if cutoff == -1
    if isfield(result, 'random')
        cutoff = prctile(result.random.sitescores{scoreID}{1},percentil);
    else
        % currently using the 99 percentil of the score as the cutoff
        cutoff = prctile(result.sitescores{scoreID}{1},percentil);
    end
end
predictions_score = toppredictions_byscorecutoff(cutoff, result, scoreID);
% the results are printed in ./result_scorecutoff.txt
outputresultintofile(['./result/result_scorecutoff', jobname, '.txt'], result, predictions_score);

%% tool for Jalview
% predictions are marked on the reference sequence which is aligned with
% the other sequences specified by seq_header

clear predictions_for_Jalview;
if predictions_selection_criterion 
    predictions_for_Jalview = predictions_top;
else
    predictions_for_Jalview = predictions_score;
end
% % show the result on the sequence after deleting the columns with too many
% % gaps
% displayresultwithjalview(result.protein, predictions_for_Jalview, ...
% seq_header); 

%% with entropy
% show the result on the sequence including all the domains selected and
% before deleting any columns
displayresultwithjalviewWithLowEntropy(result.protein, predictions_for_Jalview, seq_header);

%% show the prediction on the sequence for the whole protein rather than
%% only for the selected domains. If you have done the alignment for the
%% whole sequence rather than only do them for the selected domains.
displayresultwithjalviewWithLowEntropyOnWholeSeq(result.protein, predictions_for_Jalview, seq_header);



