%% change the HETATM of MSE into ATOM for MET, CSE to CSC
% 
% COLUMNS        DATA TYPE       CONTENTS                         
% --------------------------------------------------------------------------------
%  1 -  6        Record name     "HETATM"                                          
%  7 - 11        Integer         Atom serial number.                
% 13 - 16        Atom            Atom name                         
% 17             Character       Alternate location indicator      
% 18 - 20        Residue name    Residue name                      
% 22             Character       Chain identifier                  
% 23 - 26        Integer         Residue sequence number           
% 27             AChar           Code for insertion of residues    
% 31 - 38        Real(8.3)       Orthogonal coordinates for X      
% 39 - 46        Real(8.3)       Orthogonal coordinates for Y      
% 47 - 54        Real(8.3)       Orthogonal coordinates for Z      
% 55 - 60        Real(6.2)       Occupancy                         
% 61 - 66        Real(6.2)       Temperature factor                
% 73 - 76        LString(4)      Segment identifier, left-justified                    
% 77 - 78        LString(2)      Element symbol, right-justified  
% 79 - 80        LString(2)      Charge on the atom    
% 
% 
% COLUMNS        DATA TYPE       CONTENTS                            
% --------------------------------------------------------------------------------
%  1 -  6        Record name     "ATOM  "                                            
%  7 - 11        Integer         Atom serial number.                   
% 13 - 16        Atom            Atom name.                            
% 17             Character       Alternate location indicator.         
% 18 - 20        Residue name    Residue name.                         
% 22             Character       Chain identifier.                     
% 23 - 26        Integer         Residue sequence number.              
% 27             AChar           Code for insertion of residues.       
% 31 - 38        Real(8.3)       Orthogonal coordinates for X in Angstroms.                       
% 39 - 46        Real(8.3)       Orthogonal coordinates for Y in Angstroms.                            
% 47 - 54        Real(8.3)       Orthogonal coordinates for Z in Angstroms.                            
% 55 - 60        Real(6.2)       Occupancy.                            
% 61 - 66        Real(6.2)       Temperature factor (Default = 0.0).                   
% 73 - 76        LString(4)      Segment identifier, left-justified.   
% 77 - 78        LString(2)      Element symbol, right-justified.      
% 79 - 80        LString(2)      Charge on the atom.       
% 
% return bool == 0 no modified residue
% return bool == 1 only MSE and CSE involved
function bool = changeHETATM2ATOMinpdbForMSE(pdbfile, newpdbfile)
bool = 0;
structure = pdbread(pdbfile);
lastAtomSerID = structure.Model.Atom(size(structure.Model.Atom,2)).AtomSerNo;
fid = fopen(pdbfile);
fidnew = fopen([newpdbfile, pdbfile(size(pdbfile,2)-7:size(pdbfile,2))],'w');
while 1
    tline = fgetl(fid); if ~ischar(tline), break, end;
    if size(tline,2) > 6
        if strcmp(tline(1:6), 'HETATM') 
            if str2num(tline(7:11)) < lastAtomSerID
                if strcmpi(tline(18:20), 'MSE')
                    tline(1:6) = 'ATOM  ';
                    tline(18:20) = 'MET';
                    if strcmpi(tline(13:14), 'SE')
                        tline(77:78) = ' S';
                        tline(13:15) = ' SD';
                    end
                    bool = 1;
                elseif strcmpi(tline(18:20), 'CSE')
                    tline(1:6) = 'ATOM  ';
                    tline(18:20) =  'CYS';                    
                    if strcmpi(tline(13:14), 'SE')
                        tline(13:15) = ' SG';
                        tline(77:78) = ' S';
                    end       
                    bool = 1;
                else
                    'other modifed residues involved'
                end         
            end
        end        
    end  
    fprintf(fidnew, '%s\n', tline);
end
fclose(fid);
fclose(fidnew);



