%% input: structure and resnum of the reference sequence in MSA
% output: left_vec the id's of the columns with structure information
%         distances the structural distances between the columns with
%         structure information
function output = compute_atom_distances(structure, residue)
%% get all the side chains' coordinates
for i = 1 : size(residue.number,2)
    bool_struc(i) = 0;
    coordinates{i} = [];
    for j = 1 : size(structure.Model.Atom, 2)
        if residue.number(i) == structure.Model.Atom(j).resSeq ...
                && residue.chain(i) == structure.Model.Atom(j).chainID
            bool_struc(i) = 1;            
%             if strcmpi(structure.Model.Atom(j).resName, 'gly') && strcmpi(structure.Model.Atom(j).AtomName, 'ca')
%                 coordinates{i} = [coordinates{i}; structure.Model.Atom(j).X,...
%                     structure.Model.Atom(j).Y,...
%                     structure.Model.Atom(j).Z];
%             elseif ~strcmpi(structure.Model.Atom(j).resName, 'gly') &&  ~strcmpi(structure.Model.Atom(j).AtomName, 'ca') ...
%                     && ~strcmpi(structure.Model.Atom(j).AtomName, 'c') && ~strcmpi(structure.Model.Atom(j).AtomName, 'ca') ...
%                     && ~strcmpi(structure.Model.Atom(j).AtomName, 'o') && ~strcmpi(structure.Model.Atom(j).AtomName, 'n') ...
%                     && ~strcmpi(structure.Model.Atom(j).AtomName(1), 'h')
%                 coordinates{i} = [coordinates{i}; structure.Model.Atom(j).X,...
%                     structure.Model.Atom(j).Y,...
%                     structure.Model.Atom(j).Z];
%             end
            coordinates{i} = [coordinates{i}; structure.Model.Atom(j).X,...
                structure.Model.Atom(j).Y,...
                structure.Model.Atom(j).Z];
        end
    end
end
left_vec = [];
for i = 1 : size(residue.number,2)
    if bool_struc(i) == 1
        left_vec = [left_vec, i];
    end
end
output.left_vec = left_vec;

%% compute the distances between all the atoms and choose the closest
%% distances among the atoms of two residues as the distance between those
%% two residues
num = sum(bool_struc);
distances = zeros(num,num);
for i = 1 : num - 1
    for j = i+1 : num
        distances(i,j) = ClosestDistance(coordinates{left_vec(i)},coordinates{left_vec(j)});
        distances(j,i) = distances(i,j);
    end
end
output.distances = distances;

function distance = ClosestDistance(coordinates1,coordinates2)
n1 = size(coordinates1,1);
n2 = size(coordinates2,1);
distance = 10000;
for i = 1 : n1
    for j = 1 : n2 
        temp = norm(coordinates1(i,:) - coordinates2(j,:));
        if temp < distance
            distance = temp;
        end
    end
end


% %% side chain and main chain
% for i = 1 : size(residue.number,2)
%     bool_struc(i) = 0;
%     Sidecoordinates{i} = [];
%     Maincoordinates{i} = [];
%     for j = 1 : size(structure.Model.Atom, 2)
%         if residue.number(i) == structure.Model.Atom(j).resSeq ...
%                 && residue.chain(i) == structure.Model.Atom(j).chainID
%             bool_struc(i) = 1;  
%             
%             if strcmpi(structure.Model.Atom(j).resName, 'gly') && strcmpi(structure.Model.Atom(j).AtomName, 'ca')
%                 Sidecoordinates{i} = [Sidecoordinates{i}; structure.Model.Atom(j).X,...
%                     structure.Model.Atom(j).Y,...
%                     structure.Model.Atom(j).Z];
%             elseif ~strcmpi(structure.Model.Atom(j).resName, 'gly') &&  ~strcmpi(structure.Model.Atom(j).AtomName, 'ca') ...
%                     && ~strcmpi(structure.Model.Atom(j).AtomName, 'c') && ~strcmpi(structure.Model.Atom(j).AtomName, 'ca') ...
%                     && ~strcmpi(structure.Model.Atom(j).AtomName, 'o') && ~strcmpi(structure.Model.Atom(j).AtomName, 'n') ...
%                     && ~strcmpi(structure.Model.Atom(j).AtomName(1), 'h')
%                 Sidecoordinates{i} = [Sidecoordinates{i}; structure.Model.Atom(j).X,...
%                     structure.Model.Atom(j).Y,...
%                     structure.Model.Atom(j).Z];
%             end
%             if strcmpi(structure.Model.Atom(j).AtomName, 'ca') ...
%                     || strcmpi(structure.Model.Atom(j).AtomName, 'c') ...
%                     || strcmpi(structure.Model.Atom(j).AtomName, 'o') || strcmpi(structure.Model.Atom(j).AtomName, 'n')            
%                 Maincoordinates{i} = [Maincoordinates{i}; structure.Model.Atom(j).X,...
%                     structure.Model.Atom(j).Y,...
%                     structure.Model.Atom(j).Z];
%             end
%         end
%     end
% end
% left_vec = [];
% for i = 1 : size(residue.number,2)
%     if bool_struc(i) == 1
%         left_vec = [left_vec, i];
%     end
% end
% output.left_vec = left_vec;
% 
% %% compute the distances between all the atoms and choose the closest
% %% distances among the atoms of two residues as the distance between those
% %% two residues
% num = sum(bool_struc);
% distances = zeros(num,num);
% for i = 1 : num - 1
%     for j = i+1 : num
%         distance1 = ClosestDistance(Sidecoordinates{left_vec(i)},Maincoordinates{left_vec(j)});
%         distance2 = ClosestDistance(Maincoordinates{left_vec(i)},Sidecoordinates{left_vec(j)});
%         if distance1 < distance2
%             distances(i,j) = distance1;
%         else
%             distances(i,j) = distance2;
%         end
%         distances(j,i) = distances(i,j);
%     end
% end
% output.distances = distances;
% 
% function distance = ClosestDistance(coordinates1,coordinates2)
% n1 = size(coordinates1,1);
% n2 = size(coordinates2,1);
% distance = 10000;
% for i = 1 : n1
%     for j = 1 : n2 
%         temp = norm(coordinates1(i,:) - coordinates2(j,:));
%         if temp < distance
%             distance = temp;
%         end
%     end
% end




