%% find inter-chain contact sites in a big complex from a pdb file
% input: complex name, segment definition
% output: inter segment contact pairs and contact sites on each segment by
% residue number
% segmentdefinition.number{i} = [1, 101];  ...
%                   chain{i} = 'A';  ...
%
function output = findInterSegmentContact(workingpath, complexname, segmentdefinition)
% all segments' residues
residue.number = []; residue.chain = []; domainID = [];
numSegment = size(segmentdefinition.number,2);
for i = 1 : numSegment
    for j = 1 : size(segmentdefinition.number{i},1)
        residue.number = [residue.number,...
            segmentdefinition.number{i}(1,1):segmentdefinition.number{i}(1,2)];
        for k = segmentdefinition.number{i}(1,1):segmentdefinition.number{i}(1,2)            
            residue.chain  = [residue.chain,segmentdefinition.chain{i}];
            domainID = [domainID, i];
        end
    end
end
complexstructure = pdbread([workingpath, complexname,'.pdb']);

% distances
output1 = compute_atom_distances(complexstructure, residue);
distances = output1.distances;
left_vec = output1.left_vec;

% solvent accessability
% run joy for complex
runUnix(workingpath, ['psa ', workingpath, complexname, '.pdb']);
% joypsa -- residue and psainfo.
joypsa = readpsaJoy([workingpath, complexname, '.psa'], complexstructure);

clear protein;
protein.names = complexname;
protein.structure = complexstructure;
protein.aln.domain = segmentdefinition;
% run joy for each domain
splitPDB(workingpath, protein);
% psa for segments
for j = 1 : size(segmentdefinition.number,2)
    clear structure_info;
    structure_info = pdbread([workingpath, complexname, 'Domain', num2str(j),'.pdb']);
    runUnix(workingpath, ['psa ', workingpath, complexname, 'Domain', num2str(j), '.pdb']);
    clear domainjoypsa;
    domainjoypsa = readpsaJoy([workingpath, complexname, 'Domain', num2str(j), '.psa'], structure_info);
    for kd = 1 : size(domainjoypsa,2)
        for ka = 1 : size(joypsa,2)
            if domainjoypsa(kd).resnum == joypsa(ka).resnum ...
                    && strcmpi(domainjoypsa(kd).resname, joypsa(ka).resname) ...
                    && strcmpi(domainjoypsa(kd).reschain, joypsa(ka).reschain)
                joypsa(ka).splitedtotalside = domainjoypsa(kd).totalside;
                joypsa(ka).splitedtotalsidesum = domainjoypsa(kd).totalsidesum;
                joypsa(ka).sachange = joypsa(ka).totalside - joypsa(ka).splitedtotalside;
                joypsa(ka).sachangesum = joypsa(ka).totalsidesum - joypsa(ka).splitedtotalsidesum;
                break;
            end
        end
    end
end
% save('debug.mat','joypsa');
% check the interface between each two domains
% recording the id's in the residue vector

for i = 1 : numSegment
    for j = 1 : numSegment
        interdomaincontact{i,j} = [];
        interdomaincontactSites{i,j} = [];
        interfaceASAChange(i,j) = 0;
    end
end
'find contacts ...'

distancecutoff = 4.5;
for i = 1 : size(left_vec,2)-1
    for j = i+1 : size(left_vec,2)
        if domainID(left_vec(i)) ~= domainID(left_vec(j))
            if domainID(left_vec(i)) < domainID(left_vec(j))
                index1 = domainID(left_vec(i));
                index2 = domainID(left_vec(j));
            else
                index1 = domainID(left_vec(j));
                index2 = domainID(left_vec(i));
            end
            if distances(i,j) < distancecutoff
                interdomaincontact{index1,index2}...
                    = [interdomaincontact{index1,index2}; left_vec(i), left_vec(j)];
                if ~isin(left_vec(i), interdomaincontactSites{index1,index2})
                    interdomaincontactSites{index1,index2}...
                        = [interdomaincontactSites{index1,index2}, left_vec(i)];
                    clear temp;
                    temp = findjoypsaforresidue(joypsa, left_vec(i), residue);
                    if temp.sachangesum > 0
                        'SA in this pair increased'
                    end
                    interfaceASAChange(index1,index2)...
                        = interfaceASAChange(index1,index2) + temp.sachangesum;
                end
                if ~isin(left_vec(j), interdomaincontactSites{index1,index2})
                    interdomaincontactSites{index1,index2}...
                        = [interdomaincontactSites{index1,index2}, left_vec(j)];
                    clear temp;
                    temp = findjoypsaforresidue(joypsa, left_vec(j), residue);
                    if temp.sachangesum > 0
                        'SA in this pair increased'
                    end
                    interfaceASAChange(index1,index2)...
                        = interfaceASAChange(index1,index2) + temp.sachangesum;
                end
            end
        end
    end
end
% get the real residue
allcontactpairs.number = [];
allcontactpairs.chain = [];
for i = 1 : numSegment
    allcontactsites{i}.number = [];
    allcontactsites{i}.chain = [];
end
ASAcutoff = -10;
for i = 1 : numSegment-1
    for j = i+1 : numSegment
        if interfaceASAChange(i,j) < ASAcutoff
            for k = 1 : size(interdomaincontact{i,j},1)
                allcontactpairs.number = [allcontactpairs.number; residue.number(interdomaincontact{i,j}(k,:))];
                allcontactpairs.chain = [allcontactpairs.chain; residue.chain(interdomaincontact{i,j}(k,:))];                 
            end 
            for k = 1 : size(interdomaincontactSites{i,j},2)
                currentresidue.number =  residue.number(interdomaincontactSites{i,j}(k));
                currentresidue.chain =  residue.chain(interdomaincontactSites{i,j}(k));
                if ~isInResidueSet(currentresidue, allcontactsites{domainID(interdomaincontactSites{i,j}(k))})
                     allcontactsites{domainID(interdomaincontactSites{i,j}(k))}.number = ...
                         [allcontactsites{domainID(interdomaincontactSites{i,j}(k))}.number, currentresidue.number];
                      allcontactsites{domainID(interdomaincontactSites{i,j}(k))}.chain = ...
                          [allcontactsites{domainID(interdomaincontactSites{i,j}(k))}.chain, currentresidue.chain];
                end
            end               
        end
    end    
end

%% ASA change
for i = 1 : size(allcontactsites,2)
    for j = 1 : size(allcontactsites{i}.number,2)
        clear temp;
        temp = findjoypsaforresidue(joypsa, j, allcontactsites{i});
        allcontactsitesASAChange{i}(j) = temp.sachangesum;
    end
end

output.interfaceASAChange = interfaceASAChange; % interface asa change
output.allcontactpairs = allcontactpairs; % pairs of res number and chain id 
output.allcontactsites = allcontactsites; % sites of res number and chain id
output.interdomaincontact = interdomaincontact; % location pairs
output.interdomaincontactSites = interdomaincontactSites; % location sites
output.allcontactsitesASAChange = allcontactsitesASAChange; % asa change for each contact site
output.joypsa = joypsa;  % all the asa information for all residues














