%% intra-domain neighbor
% 'sequence', 'structure', neighbor
% AA composition score for pairs
% average pPairScore between patches
function score = pPairScore(alignment, protein, stats, proteinname, temppath)
% pcpair = [1.2711, 0.9624, 2.1402, 0.8014];
pcpair = stats.Pairpropensity{3};
aminonames = {'R', 'N', 'D', 'C', 'Q', 'E', 'G', 'H', 'I', 'L', 'K', 'M', 'F', 'P', 'S', 'T', 'W', 'Y', 'V', 'A'};
seprate_site = protein.aln.nogap.seprate_site;
num_seq = size(alignment,1);
num_column = size(alignment(1,1).Sequence, 2);
mapped_index = -1 * ones(num_seq, num_column);
fiddata = fopen([temppath, 'IndexMSA', proteinname,'.mat']);
if fiddata == -1    
    for i = 1 : num_seq
        for j = 1 : num_column
            mapped_index(i,j) = MapResiduesToIndex(alignment(i,1).Sequence(j), aminonames);
        end
    end
    save([temppath, 'IndexMSA', proteinname,'.mat'], 'mapped_index');
else
    fclose(fiddata);
    clear tempdata;
    tempdata = load([temppath, 'IndexMSA', proteinname,'.mat']);
    mapped_index = tempdata.mapped_index;
    clear tempdata;    
end    
numAAtypes = 7;
featureScoreForPair = zeros(numAAtypes,numAAtypes);
for i = 1 : numAAtypes
    for j = 1 : numAAtypes
        featureScoreForPair(i,j) = pcpair( ClassifyTypes( [i,j] ) );
    end
end

%% compute score
temp = zeros(1,seprate_site*(num_column-seprate_site));
index = zeros(seprate_site*(num_column-seprate_site),2);
t = 1;
for j = 1 : seprate_site
    for k = seprate_site+1 : num_column
        index(t,:) = [j,k];
        t = t + 1;
    end
end
parfor t = 1 : size(index,1)
    temp(t) = AApairneighborhoodscore(mapped_index(:,index(t,1)),...
        mapped_index(:,index(t,2)), featureScoreForPair);
end

score =  zeros(num_column, num_column);
t = 1;
for j = 1 : seprate_site
    for k = seprate_site+1 : num_column
        score(j,k) = temp(t);
        t = t + 1;
        score(k,j) = score(j,k);
    end
end

function neighborhoodscore = AApairneighborhoodscore(aaindexJ, aaindexK, featureScoreForPair)
%% average AApair propensity scores among all possible pairs of residues
%% between two patches defined by intra-domain structure neighbor for each
%% site
summ = 0; count = 0;
% sum over all possible pairs of sites between two patches J and K
for k = 1 : size(aaindexJ,1)
    if aaindexJ(k,1) > 0 && aaindexK(k,1) > 0
        summ = summ +  featureScoreForPair(aaindexJ(k,1),aaindexK(k,1));
        count = count + 1;
    end
end
neighborhoodscore = summ / count;




