%% intra-domain neighbor
% 'sequence', 'structure', neighbor
% AA composition score for pairs
% average pTriangleScore between patches
function score = pTriangleScoreNeighbor(alignment, protein, Ntype, stats, proteinname, temppath, EBindex, parameters)

seprate_site = protein.aln.nogap.seprate_site;
aminonames = {'R', 'N', 'D', 'C', 'Q', 'E', 'G', 'H', 'I', 'L', 'K', 'M', 'F', 'P', 'S', 'T', 'W', 'Y', 'V', 'A'};
num_seq = size(alignment,1);
num_column = size(alignment(1,1).Sequence, 2);

mapped_index = -1 * ones(num_seq, num_column);
fiddata = fopen([temppath, 'IndexMSA', proteinname,'.mat']);
if fiddata == -1    
    for i = 1 : num_seq
        for j = 1 : num_column
            mapped_index(i,j) = MapResiduesToIndex(alignment(i,1).Sequence(j), aminonames);
        end
    end
    save([temppath, 'IndexMSA', proteinname,'.mat'], 'mapped_index');
else
    fclose(fiddata);
    clear tempdata;
    tempdata = load([temppath, 'IndexMSA', proteinname,'.mat']);
    mapped_index = tempdata.mapped_index;
    clear tempdata;    
end 

pctriangle = stats.Trianglepropensity{3};
% pctriangle = [0,0,0,0,0,0,0,0,0,0,0,0,1.5528,0.7402,0.9533,1.4994,1.8151,0.8787,0,0.6415;];
% pctwostar = [0,0,0.9928,0,0.8127,0.9813,1.8703,0.8145,1.6881,0.6269;];
% pcpair = [1.2711, 0.9624, 2.1402, 0.8014];
pcpair = stats.Pairpropensity{3};
aapairweight = stats.neighborhood_pair{3};
% groupTable defined by the AA category
numAAtypes = 7;
featureScoreForPair = zeros(numAAtypes,numAAtypes);
for i = 1 : numAAtypes
    for j = 1 : numAAtypes
        featureScoreForPair(i,j) = pcpair( ClassifyTypes( [i,j] ) );
    end
end

%%neighbor information
fiddata = fopen([temppath, 'NeighborInfo', proteinname, '.mat']);
if fiddata == -1
    for j = 1 : num_column
        neighborLocationForColumn{j} = [];
        switch lower(Ntype)
            case 'sequence'
                % neighbors on sequence
                % it's a gap or the residue without structure information avaliable
                % use the sequence neighbors
                % neighbors on sequence
                if j <= seprate_site
                    neighborLocationForColumn{j} =  GetSequenceNeighborLoction(j, 1, seprate_site);
                else
                    neighborLocationForColumn{j} =  GetSequenceNeighborLoction(j,  1+seprate_site, num_column);
                end
            case 'structure'
                % neighbor in space
                % 4.5A
                % if there is no structure information avaliable then we use
                % 2 sequence neighbors
                neighborLocationForColumn{j}  = GetStructureNeighbor(j, protein, parameters.discutoff, EBindex);
            otherwise
                'wrong neighbor type'
        end
    end
    save([temppath, 'NeighborInfo', proteinname, '.mat'], 'neighborLocationForColumn');    
else
    fclose(fiddata);
    clear tempdata;
    tempdata = load([temppath, 'NeighborInfo', proteinname, '.mat']);
    neighborLocationForColumn = tempdata.neighborLocationForColumn;
    clear tempdata;
end

%% compute score
temp = zeros(1,seprate_site*(num_column-seprate_site));
index = zeros(seprate_site*(num_column-seprate_site),2);
t = 1;
for j = 1 : seprate_site
    for k = seprate_site+1 : num_column
        index(t,:) = [j,k];
        t = t + 1;
    end
end
parfor t = 1 : size(index,1)
    neighborLocationForJ = [];
    neighborLoctaionForK = [];
    neighborLocationForJ = neighborLocationForColumn{index(t,1)};
    neighborLoctaionForK = neighborLocationForColumn{index(t,2)};
    temp(t) = AATriangleneighborhoodscore(mapped_index, neighborLocationForJ, neighborLoctaionForK, ...
        pctriangle, featureScoreForPair, aapairweight, index(t,:));
end

score =  zeros(num_column, num_column);
t = 1;
for j = 1 : seprate_site
    for k = seprate_site+1 : num_column
        score(j,k) = temp(t);
        t = t + 1;
        score(k,j) = score(j,k);
    end
end



function triangleScore = AATriangleneighborhoodscore(aaindex, neighborJ, neighborK, pctriangle, featureScoreForPair, aapairweight, oindex)

neighborall = [neighborJ, neighborK];
nei_num = size(neighborall,2);
pairgroup = ClassifyTypes(aaindex(:,oindex));
if  nei_num  < 2
    'error in neighbor identification'
elseif nei_num == 2
    t = 0;
    summ = 0;
    for k = 1 : size(aaindex,1)
        if aaindex(k,neighborJ) > 0 && aaindex(k,neighborK) > 0
            summ = summ + featureScoreForPair(aaindex(k,neighborJ), aaindex(k,neighborK));
            t = t + 1;
        end
    end
    triangleScore = summ / t;
else    
    summ = 0; t = 0;
    for i = 1 : nei_num
        clear trigroup;
        if neighborall(i) ~= oindex(1) && neighborall(i) ~= oindex(2)
            trigroup = ClassifyTypes(aaindex(:, [oindex, neighborall(i)]));
            for k = 1 : size(aaindex,1)
                if trigroup(k) > 0 && aaindex(k, neighborall(i)) > 0 && aaindex(k, neighborall(i)) > 0
                    t = t + 1;
                    summ = summ + pctriangle(trigroup(k)) * aapairweight(pairgroup(k), aaindex(k, neighborall(i)));
                end
            end
        end
    end
    triangleScore = summ / t;
end
